from typing import List, Optional

from pydantic import BaseModel

from ..node import Node
from ..pagination import Pagination
from ..thread import Thread

__all__ = (
    "ForumGetResponse",
    "ForumMarkReadResponse",
    "ForumThreadsGetResponse",
)


class ForumGetResponse(BaseModel):
    forum: Node
    threads: Optional[List[Thread]] = None
    pagination: Optional[Pagination] = None
    sticky: Optional[List[Thread]] = None


class ForumMarkReadResponse(BaseModel):
    success: bool


class ForumThreadsGetResponse(BaseModel):
    threads: List[Thread]
    pagination: Pagination
    sticky: Optional[List[Thread]] = None



import logging
from dataclasses import dataclass
from typing import Optional

__all__ = ("LoggerData", "setup_logging")


@dataclass
class LoggerData:
    handler: Optional[logging.Handler] = None
    formatter: Optional[logging.Formatter] = None
    level: Optional[int] = None
    root: bool = True


def setup_logging(data: Optional[LoggerData]) -> None:
    level = data.level if data else None
    handler = data.handler if data else None
    formatter = data.formatter if data else None
    root = data.root if data else None

    if level is None:
        level = logging.INFO

    if handler is None:
        handler = logging.StreamHandler()

    if formatter is None:
        dt_fmt = "%Y-%m-%d %H:%M:%S"
        formatter = logging.Formatter(
            "[{asctime}] [{levelname:<8}] {name}: {message}",
            dt_fmt,
            style="{",
        )

    if root:
        logger = logging.getLogger()
    else:
        library = __name__
        logger = logging.getLogger(library)

    handler.setFormatter(formatter)
    logger.setLevel(level)
    logger.addHandler(handler)



from typing import Dict, List, Optional

from pydantic import BaseModel, field_serializer

from ..groups import ArzGuardGroupsIdsEnum
from .user import DateOfBirth, Option, Privacy, Profile

__all__ = (
    "UserCreateParams",
    "UserDeleteParams",
    "UserDemoteParams",
    "UserGetParams",
    "UserProfilePostsGetParams",
    "UserPromoteParams",
    "UserUpdateParams",
    "UsersFindEmailParams",
    "UsersFindNameParams",
    "UsersGetParams",
)


class UsersGetParams(BaseModel):
    page: Optional[int] = None


class UserCreateParams(BaseModel):
    username: str
    email: str
    password: str
    option: Optional[Option] = None
    profile: Optional[Profile] = None
    privacy: Optional[Privacy] = None
    visible: Optional[bool] = None
    activity_visible: Optional[bool] = None
    timezone: Optional[str] = None
    custom_title: Optional[str] = None
    user_group_id: Optional[int] = None
    secondary_group_ids: Optional[List[int]] = None
    user_state: Optional[str] = None
    is_staff: Optional[bool] = None
    message_count: Optional[int] = None
    reaction_score: Optional[int] = None
    trophy_points: Optional[int] = None
    username_change_visible: Optional[bool] = None
    dob: Optional[Dict[str, int]] = None
    custom_fields: Optional[str] = None

    @field_serializer(
        "visible", "activity_visible", "is_staff", "username_change_visible"
    )
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class UsersFindEmailParams(BaseModel):
    email: str


class UsersFindNameParams(BaseModel):
    username: str


class UserGetParams(BaseModel):
    with_posts: Optional[bool] = None
    page: Optional[int] = None

    @field_serializer("with_posts")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class UserUpdateParams(BaseModel):
    option: Optional[Option] = None
    profile: Optional[Profile] = None
    privacy: Optional[Privacy] = None
    visible: Optional[bool] = None
    activity_visible: Optional[bool] = None
    timezone: Optional[str] = None
    custom_title: Optional[str] = None
    username: Optional[str] = None
    email: Optional[str] = None
    password: Optional[str] = None
    user_group_id: Optional[int] = None
    secondary_group_ids: Optional[List[int]] = None
    user_state: Optional[str] = None
    is_staff: Optional[bool] = None
    message_count: Optional[int] = None
    reaction_score: Optional[int] = None
    trophy_points: Optional[int] = None
    username_change_visible: Optional[bool] = None
    dob: Optional[DateOfBirth] = None
    custom_fields: Optional[str] = None

    @field_serializer(
        "visible", "activity_visible", "is_staff", "username_change_visible"
    )
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class UserDeleteParams(BaseModel):
    rename_to: str


class UserProfilePostsGetParams(BaseModel):
    page: int


class UserDemoteParams(BaseModel):
    group: ArzGuardGroupsIdsEnum

    @field_serializer("group")
    def serialize_group(self, v: ArzGuardGroupsIdsEnum):
        return v.value


class UserPromoteParams(BaseModel):
    group: ArzGuardGroupsIdsEnum

    @field_serializer("group")
    def serialize_group(self, v: ArzGuardGroupsIdsEnum):
        return v.value



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from pydantic import BaseModel, Field

from ...types.discussion_state import DiscussionStateEnum
from ..user import User
from ..vote_type import VoteTypeEnum

if TYPE_CHECKING:
    from ..node import Node

__all__ = ("Thread",)


class Thread(BaseModel):
    thread_id: int
    node_id: int
    title: str
    username: str
    user_id: int
    user: User = Field(alias="User")
    is_watching: bool | None = None
    visitor_post_count: int | None = None
    is_unread: bool | None = None
    custom_fields: dict[str, Any]
    tags: list[str] | None = None
    prefix: str | None = None
    can_edit: bool
    can_edit_tags: bool
    can_reply: bool
    can_soft_delete: bool
    can_hard_delete: bool
    can_view_attachments: bool
    view_url: str
    is_first_post_pinned: bool
    highlighted_post_ids: list[int]
    is_search_engine_indexable: bool | None = None
    index_state: str | None = None
    forum: Node | None = Field(alias="Forum", default=None)
    vote_score: int | None = None
    can_content_vote: bool | None = None
    allowed_content_vote_types: list[str] | None = None
    is_content_voted: bool | None = None
    visitor_content_vote: VoteTypeEnum | None = None
    reply_count: int
    view_count: int
    post_date: int
    sticky: bool
    discussion_state: DiscussionStateEnum
    discussion_open: bool
    discussion_type: str
    first_post_id: int
    last_post_date: int
    last_post_id: int
    last_post_user_id: int
    last_post_username: str
    first_post_reaction_score: int
    prefix_id: int



from typing import Any, Dict, Optional

from pydantic import BaseModel, field_serializer

from nightforo.types.content_type import ContentTypeEnum

__all__ = (
    "AttachmentUploadParams",
    "AttachmentsCreateNewKeyParams",
    "AttachmentsGetParams",
)


class AttachmentsGetParams(BaseModel):
    key: str


class AttachmentUploadParams(BaseModel):
    key: str


class AttachmentsCreateNewKeyParams(BaseModel):
    type: ContentTypeEnum
    context: Optional[Dict[str, Any]] = None

    @field_serializer("type")
    def serialize_type(self, v: ContentTypeEnum):
        return v.value



from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel

from nightforo.types.forum.forum import LinkForumTypeData

from ...types.forum import (
    ForumTypeData,
    SearchForumTypeData,
)
from ...types.node_type import NodeTypeEnum
from ...types.page import PageTypeData

AnyNodeTypeData = Union[
    ForumTypeData,
    SearchForumTypeData,
    PageTypeData,
    LinkForumTypeData,
    Dict[str, Any],
]

__all__ = ("Breadcrumb", "Node")


class Breadcrumb(BaseModel):
    node_id: Optional[int] = None
    title: Optional[str] = None
    node_type_id: Optional[NodeTypeEnum] = None


class Node(BaseModel):
    node_id: int

    title: str
    node_name: Optional[str] = None
    node_type_id: NodeTypeEnum
    breadcrumbs: List[Breadcrumb]
    type_data: Union[
        AnyNodeTypeData,
        Dict[str, Any],
    ]
    view_url: str
    description: str
    parent_node_id: int
    display_order: int
    display_in_list: bool



from enum import Enum

__all__ = ("APIScopeIdsEnum",)


class APIScopeIdsEnum(Enum):
    ALERT_READ = "alert:read"
    ALERT_WRITE = "alert:write"
    ATTACHMENT_DELETE = "attachment:delete"
    ATTACHMENT_READ = "attachment:read"
    ATTACHMENT_WRITE = "attachment:write"
    AUTH = "auth"
    AUTH_LOGIN_TOKEN = "auth:login_token"
    CONVERSATION_READ = "conversation:read"
    CONVERSATION_WRITE = "conversation:write"
    NODE_DELETE = "node:delete"
    NODE_WRITE = "node:write"
    NODE_READ = "node:read"
    PROFILE_POST_DELETE_HARD = "profile_post:delete_hard"
    PROFILE_POST_READ = "profile_post:read"
    PROFILE_POST_WRITE = "profile_post:write"
    THREAD_DELETE_HARD = "thread:delete_hard"
    THREAD_READ = "thread:read"
    THREAD_WRITE = "thread:write"
    USER_DELETE = "user:delete"
    USER_WRITE = "user:write"
    USER_READ = "user:read"
    USER_PROMOTE = "user:promote"



from pydantic import BaseModel

__all__ = ("Attachment",)


class Attachment(BaseModel):
    filename: str
    file_size: int
    height: int
    width: int
    thumbnail_url: str
    direct_url: str
    is_video: bool
    is_audio: bool
    attachment_id: int
    content_type: str
    content_id: int
    attach_date: int
    view_count: int



from pydantic import BaseModel

from ...types.post_react_state import PostReactStateEnum
from . import ProfilePostComment

__all__ = (
    "ProfilePostCommentCreateResponse",
    "ProfilePostCommentDeleteResponse",
    "ProfilePostCommentGetResponse",
    "ProfilePostCommentReactResponse",
    "ProfilePostCommentUpdateResponse",
)


class ProfilePostCommentCreateResponse(BaseModel):
    success: bool
    comment: ProfilePostComment


class ProfilePostCommentGetResponse(BaseModel):
    comment: ProfilePostComment


class ProfilePostCommentUpdateResponse(BaseModel):
    success: bool
    comment: ProfilePostComment


class ProfilePostCommentDeleteResponse(BaseModel):
    success: bool


class ProfilePostCommentReactResponse(BaseModel):
    success: bool
    action: PostReactStateEnum



from pydantic import BaseModel

from ..user import User

__all__ = (
    "MeAvatarDeleteResponse",
    "MeAvatarUpdateResponse",
    "MeEmailUpdateResponse",
    "MeGetResponse",
    "MePasswordUpdateResponse",
    "MeUpdateResponse",
)


class MeGetResponse(BaseModel):
    me: User


class MeUpdateResponse(BaseModel):
    success: bool


class MeAvatarUpdateResponse(BaseModel):
    success: bool


class MeAvatarDeleteResponse(BaseModel):
    success: bool


class MeEmailUpdateResponse(BaseModel):
    success: bool
    confirmation_required: bool


class MePasswordUpdateResponse(BaseModel):
    success: bool



from typing import Dict, List, Optional, Union

from pydantic import BaseModel, field_validator

from ..groups import ArzGuardGroupsIdsEnum, ArzGuardGroupsNamesEnum
from ..pagination import Pagination
from ..profile_post import ProfilePost
from .user import User

__all__ = (
    "DemoteUserResponse",
    "GetDemoteGroupsResponse",
    "GetPromoteGroupsResponse",
    "PromoteUserResponse",
    "UserAvatarDeleteResponse",
    "UserAvatarUpdateResponse",
    "UserCreateResponse",
    "UserDeleteResponse",
    "UserFindEmailResponse",
    "UserFindNameResponse",
    "UserGetResponse",
    "UserProfilePostsGetResponse",
    "UserUpdateResponse",
    "UsersGetResponse",
)


class UsersGetResponse(BaseModel):
    users: List[User]
    pagination: Pagination


class UserCreateResponse(BaseModel):
    success: bool
    user: User


class UserFindEmailResponse(BaseModel):
    user: Optional[User] = None


class UserFindNameResponse(BaseModel):
    exact: Optional[User] = None
    recommendations: List[User]


class UserGetResponse(BaseModel):
    user: User
    profile_posts: Optional[List[ProfilePost]] = None
    pagination: Optional[Pagination] = None


class UserUpdateResponse(BaseModel):
    success: bool
    user: User


class UserDeleteResponse(BaseModel):
    success: bool


class UserAvatarUpdateResponse(BaseModel):
    success: bool


class UserAvatarDeleteResponse(BaseModel):
    success: bool


class UserProfilePostsGetResponse(BaseModel):
    profile_posts: List[ProfilePost]
    pagination: Pagination


class GetDemoteGroupsResponse(BaseModel):
    success: Optional[bool] = None
    groups: Dict[ArzGuardGroupsIdsEnum, ArzGuardGroupsNamesEnum]


class GetPromoteGroupsResponse(BaseModel):
    success: Optional[bool] = None
    groups: Dict[ArzGuardGroupsIdsEnum, ArzGuardGroupsNamesEnum]


class PromoteUserResponse(BaseModel):
    success: bool
    groups: Dict[ArzGuardGroupsIdsEnum, ArzGuardGroupsNamesEnum]
    user: Optional[User] = None

    @field_validator("groups", mode="before")
    @classmethod
    def validate_groups(
        cls,
        v: Union[
            Dict[str, ArzGuardGroupsNamesEnum],
            List[ArzGuardGroupsNamesEnum],
        ],
    ) -> Dict[ArzGuardGroupsIdsEnum, ArzGuardGroupsNamesEnum]:
        if isinstance(v, list):
            v = {}

        return {ArzGuardGroupsIdsEnum(int(k)): v for k, v in v.items()}


class DemoteUserResponse(BaseModel):
    success: bool
    groups: Dict[ArzGuardGroupsIdsEnum, ArzGuardGroupsNamesEnum]
    user: Optional[User] = None

    @field_validator("groups", mode="before")
    @classmethod
    def validate_groups(
        cls,
        v: Union[
            Dict[str, ArzGuardGroupsNamesEnum],
            List[ArzGuardGroupsNamesEnum],
        ],
    ) -> Dict[ArzGuardGroupsIdsEnum, ArzGuardGroupsNamesEnum]:
        if isinstance(v, list):
            v = {}

        return {ArzGuardGroupsIdsEnum(int(k)): v for k, v in v.items()}



__version__ = "1.1.7"

from .client import *  # noqa: F403
from .errors import *  # noqa: F403
from .types.alert import *  # noqa: F403
from .types.api_key_type import *  # noqa: F403
from .types.api_scopes import *  # noqa: F403
from .types.attachment import *  # noqa: F403
from .types.auth import *  # noqa: F403
from .types.conversation import *  # noqa: F403
from .types.conversation_message import *  # noqa: F403
from .types.direction import *  # noqa: F403
from .types.discussion_state import *  # noqa: F403
from .types.file import *  # noqa: F403  # noqa: F403
from .types.forum import *  # noqa: F403
from .types.forum_type import *  # noqa: F403
from .types.groups import *  # noqa: F403
from .types.me import *  # noqa: F403
from .types.node import *  # noqa: F403
from .types.node_type import *  # noqa: F403
from .types.page import *  # noqa: F403
from .types.pagination import *  # noqa: F403  # noqa: F403
from .types.post import *  # noqa: F403
from .types.post_react_state import *  # noqa: F403
from .types.profile_post import *  # noqa: F403
from .types.profile_post_comment import *  # noqa: F403
from .types.stats import *  # noqa: F403
from .types.thread import *  # noqa: F403
from .types.thread_type import *  # noqa: F403
from .types.user import *  # noqa: F403
from .types.vote_type import *  # noqa: F403
from .utils.logger import *  # noqa: F403

Post.model_rebuild()  # noqa: F405
Thread.model_rebuild()  # noqa: F405
UserGetResponse.model_rebuild()  # noqa: F405
ProfilePostGetResponse.model_rebuild()  # noqa: F405
ProfilePostCommentGetResponse.model_rebuild()  # noqa: F405



from typing import Dict, Optional

from pydantic import BaseModel, ConfigDict, field_serializer

from ..user import Option, Privacy, Profile

__all__ = (
    "MeEmailUpdateParams",
    "MePasswordUpdateParams",
    "MeUpdateParams",
)


class MeUpdateParams(BaseModel):
    option: Optional[Option] = None
    profile: Optional[Profile] = None
    privacy: Optional[Privacy] = None
    visible: Optional[bool] = None
    activity_visible: Optional[bool] = None
    timezone: Optional[str] = None
    custom_title: Optional[str] = None
    custom_fields: Optional[Dict[str, str]] = None

    model_config = ConfigDict(arbitrary_types_allowed=True)

    @field_serializer("visible", "activity_visible")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class MeEmailUpdateParams(BaseModel):
    current_password: str
    email: str


class MePasswordUpdateParams(BaseModel):
    current_password: str
    new_password: str



from typing import Optional

from pydantic import BaseModel, field_serializer

__all__ = (
    "AlertMarkParams",
    "AlertSendParams",
    "AlertsGetParams",
    "AlertsMarkAllParams",
)


class AlertsGetParams(BaseModel):
    page: Optional[int] = None
    cutoff: Optional[int] = None
    unviewed: Optional[bool] = None
    unread: Optional[bool] = None

    @field_serializer("unviewed", "unread")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class AlertSendParams(BaseModel):
    to_user_id: int
    alert: str
    from_user_id: Optional[int] = None
    link_url: Optional[str] = None
    link_title: Optional[str] = None


class AlertsMarkAllParams(BaseModel):
    read: Optional[bool] = None
    viewed: Optional[bool] = None

    @field_serializer("read", "viewed")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class AlertMarkParams(BaseModel):
    read: Optional[bool] = None
    unread: Optional[bool] = None
    viewed: Optional[bool] = None

    @field_serializer("read", "unread", "viewed")
    def serialize_bool(self, v: bool):
        return 1 if v else 0



from typing import Optional

from pydantic import BaseModel

from ...types.post_react_state import PostReactStateEnum
from .post import Post

__all__ = (
    "PostCreateResponse",
    "PostDeleteResponse",
    "PostGetResponse",
    "PostMarkSolutionResponse",
    "PostReactResponse",
    "PostUpdateResponse",
    "PostVoteResponse",
)


class PostCreateResponse(BaseModel):
    success: bool
    post: Post


class PostGetResponse(BaseModel):
    post: Post


class PostUpdateResponse(BaseModel):
    success: bool
    post: Post


class PostDeleteResponse(BaseModel):
    success: bool


class PostMarkSolutionResponse(BaseModel):
    success: bool
    new_solution_post: Optional[Post] = None
    old_solution_post: Optional[Post] = None


class PostReactResponse(BaseModel):
    success: bool
    action: PostReactStateEnum


class PostVoteResponse(BaseModel):
    success: bool
    action: PostReactStateEnum



from enum import Enum
from typing import List, Optional, Union

from pydantic import BaseModel, Field

from ...types.groups import ArzGuardGroupsIdsEnum

__all__ = (
    "DateOfBirth",
    "Option",
    "Privacy",
    "Profile",
    "ProfileAvatars",
    "ProfileBanners",
    "ProfilePrivacyLevelEnum",
    "User",
    "UserCustomFields",
    "UserStateEnum",
)


class DateOfBirth(BaseModel):
    year: Optional[int] = None
    month: Optional[int] = None
    day: Optional[int] = None


class ProfileAvatars(BaseModel):
    o: Optional[str] = None
    h: Optional[str] = None
    l: Optional[str] = None  # noqa: E741
    m: Optional[str] = None
    s: Optional[str] = None


class ProfileBanners(BaseModel):
    l: Optional[str] = None  # noqa: E741
    m: Optional[str] = None


class UserCustomFields(BaseModel):
    server: Optional[str] = None
    bot_owner: Optional[str] = None
    gender: Optional[str] = None
    occupation: Optional[str] = None
    vk: Optional[str] = None
    discord: Optional[str] = None
    telegram: Optional[str] = Field(default=None, alias="Telegram")


class Option(BaseModel):
    creation_watch_state: Optional[str] = None
    interaction_watch_state: Optional[str] = None
    content_show_signature: Optional[bool] = None
    email_on_conversation: Optional[bool] = None
    push_on_conversation: Optional[bool] = None
    receive_admin_email: Optional[bool] = None
    show_dob_year: Optional[bool] = None
    show_dob_date: Optional[bool] = None


class Profile(BaseModel):
    location: Optional[str] = None
    website: Optional[str] = None
    about: Optional[str] = None
    signature: Optional[str] = None


class Privacy(BaseModel):
    allow_view_profile: Optional[str] = None
    allow_post_profile: Optional[str] = None
    allow_receive_news_feed: Optional[str] = None
    allow_send_personal_conversation: Optional[str] = None
    allow_view_identities: Optional[str] = None


class UserStateEnum(Enum):
    VALID = "valid"
    EMAIL_CONFIRM = "email_confirm"
    EMAIL_CONFIRM_EDIT = "email_confirm_edit"
    EMAIL_BOUNCE = "email_bounce"
    MODERATED = "moderated"
    REJECTED = "rejected"
    DISABLED = "disabled"


class ProfilePrivacyLevelEnum(Enum):
    MEMBERS = "members"
    EVERYONE = "everyone"
    NONE = "none"


class User(BaseModel):
    user_id: int
    username: str

    activity_visible: Optional[bool] = None
    age: Optional[int] = None
    alert_optout: Optional[List[str]] = None
    allow_post_profile: Optional[ProfilePrivacyLevelEnum] = None
    allow_receive_news_feed: Optional[ProfilePrivacyLevelEnum] = None
    allow_send_personal_conversation: Optional[ProfilePrivacyLevelEnum] = None
    allow_view_identities: Optional[ProfilePrivacyLevelEnum] = None
    allow_view_profile: Optional[ProfilePrivacyLevelEnum] = None
    avatar_urls: Optional[ProfileAvatars] = None
    profile_banner_urls: Optional[ProfileBanners] = None
    can_ban: bool
    can_converse: bool
    can_edit: bool
    can_follow: bool
    can_ignore: bool
    can_post_profile: bool
    can_view_profile: bool
    can_view_profile_posts: bool
    can_warn: bool
    content_show_signature: Optional[bool] = None
    creation_watch_state: Optional[str] = None
    custom_fields: Optional[UserCustomFields] = None
    custom_title: Optional[str] = None
    dob: Optional[DateOfBirth] = None
    email: Optional[str] = None
    email_on_conversation: Optional[bool] = None
    gravatar: Optional[str] = None
    interaction_watch_state: Optional[str] = None
    is_admin: Optional[bool] = None
    is_banned: Optional[bool] = None
    is_discouraged: Optional[bool] = None
    is_followed: Optional[bool] = None
    is_ignored: Optional[bool] = None
    is_moderator: Optional[bool] = None
    is_super_admin: Optional[bool] = None
    last_activity: Optional[int] = None
    location: Optional[str] = None
    push_on_conversation: Optional[bool] = None
    push_optout: Optional[List[str]] = None
    receive_admin_email: Optional[bool] = None
    secondary_group_ids: Optional[List[ArzGuardGroupsIdsEnum]] = None
    show_dob_date: Optional[bool] = None
    show_dob_year: Optional[bool] = None
    signature: Optional[str] = None
    timezone: Optional[str] = None
    use_tfa: Optional[bool] = None
    user_group_id: Optional[int] = None
    user_state: Optional[UserStateEnum] = None

    user_title: Optional[Union[str, bool]] = None

    visible: Optional[bool] = None
    warning_points: Optional[int] = None
    website: Optional[str] = None
    view_url: str
    message_count: int
    question_solution_count: int
    register_date: int
    trophy_points: int
    is_staff: bool
    reaction_score: int
    vote_score: int



from typing import Optional

from pydantic import BaseModel, field_serializer

from ...types.direction import DirectionTypeEnum

__all__ = (
    "ProfilePostCreateParams",
    "ProfilePostDeleteParams",
    "ProfilePostGetParams",
    "ProfilePostReactParams",
    "ProfilePostUpdateParams",
)


class ProfilePostCreateParams(BaseModel):
    user_id: int
    message: str
    attachment_key: Optional[str] = None


class ProfilePostGetParams(BaseModel):
    with_comments: Optional[bool] = None
    page: Optional[int] = None
    direction: Optional[DirectionTypeEnum] = None

    @field_serializer("with_comments")
    def serialize_bool(self, v: bool):
        return 1 if v else 0

    @field_serializer("direction")
    def serialize_type(self, v: DirectionTypeEnum):
        return v.value


class ProfilePostUpdateParams(BaseModel):
    message: Optional[str] = None
    author_alert: Optional[bool] = None
    author_alert_reason: Optional[str] = None
    attachment_key: Optional[str] = None

    @field_serializer("author_alert")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ProfilePostDeleteParams(BaseModel):
    hard_delete: Optional[bool] = None
    reason: Optional[str] = None
    author_alert: Optional[bool] = None
    author_alert_reason: Optional[str] = None

    @field_serializer("hard_delete", "author_alert")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ProfilePostReactParams(BaseModel):
    reaction_id: int



from enum import Enum
from typing import Optional, Union

from pydantic import BaseModel, field_serializer

from ...types.direction import DirectionTypeEnum
from ...types.thread_type import ThreadTypeEnum

__all__ = (
    "ForumGetParams",
    "ForumMarkReadParams",
    "ForumThreadsGetParams",
    "OrderField",
)


class OrderField(Enum):
    LAST_POST_DATE = "last_post_date"
    POST_DATE = "post_date"
    TITLE = "title"
    REPLY_COUNT = "reply_count"
    VIEW_COUNT = "view_count"
    VOTE_SCORE = "vote_score"
    FIRST_POST_REACTION_SCORE = "first_post_reaction_score"


class ForumGetParams(BaseModel):
    with_threads: Optional[bool] = None
    page: Optional[int] = None
    prefix_id: Optional[int] = None
    starter_id: Optional[int] = None
    last_days: Optional[int] = None
    unread: Optional[bool] = None
    thread_type: Optional[ThreadTypeEnum] = None
    order: Optional[OrderField] = None
    direction: Optional[DirectionTypeEnum] = None

    @field_serializer("with_threads", "unread")
    def serialize_bool(self, v: bool):
        return 1 if v else 0

    @field_serializer("direction", "thread_type", "order")
    def serialize_type(
        self, v: Union[ThreadTypeEnum, DirectionTypeEnum, OrderField]
    ):
        return v.value


class ForumMarkReadParams(BaseModel):
    date: int


class ForumThreadsGetParams(BaseModel):
    page: Optional[int] = None
    prefix_id: Optional[int] = None
    starter_id: Optional[int] = None
    last_days: Optional[int] = None
    unread: Optional[bool] = None
    thread_type: Optional[str] = None
    order: Optional[str] = None
    direction: Optional[DirectionTypeEnum] = None

    @field_serializer("unread")
    def serialize_bool(self, v: bool):
        return 1 if v else 0

    @field_serializer("direction")
    def serialize_type(self, v: DirectionTypeEnum):
        return v.value



from enum import Enum

__all__ = ("ThreadTypeEnum",)


class ThreadTypeEnum(Enum):
    DISCUSSION = "discussion"
    REDIRECT = "redirect"
    QUESTION = "question"
    POLL = "poll"
    SUGGESTION = "suggestion"
    ARTICLE = "article"



from typing import Dict, List

from pydantic import BaseModel

from .node import Node

__all__ = (
    "NodeCreateResponse",
    "NodeDeleteResponse",
    "NodeGetResponse",
    "NodeUpdateResponse",
    "NodesFlattenedGetResponse",
    "NodesGetResponse",
)


class NodesGetResponse(BaseModel):
    tree_map: Dict[int, List[int]]
    nodes: List[Node]


class NodeCreateResponse(BaseModel):
    node: Node


class NodesFlat(BaseModel):
    node: Node
    depth: int


class NodesFlattenedGetResponse(BaseModel):
    nodes_flat: List[NodesFlat]


class NodeGetResponse(BaseModel):
    node: Node


class NodeUpdateResponse(BaseModel):
    node: Node


class NodeDeleteResponse(BaseModel):
    success: bool



from typing import BinaryIO, List, Optional

from pydantic import BaseModel, ConfigDict

from .attachment import Attachment

__all__ = (
    "AttachmentDeleteResponse",
    "AttachmentGetDataResponse",
    "AttachmentGetResponse",
    "AttachmentGetThumbnailResponse",
    "AttachmentUploadResponse",
    "AttachmentsCreateNewKeyResponse",
    "AttachmentsGetResponse",
)


class AttachmentsGetResponse(BaseModel):
    attachments: List[Attachment]


class AttachmentUploadResponse(BaseModel):
    attachment: Attachment


class AttachmentsCreateNewKeyResponse(BaseModel):
    key: str
    attachment: Optional[Attachment]


class AttachmentGetResponse(BaseModel):
    attachment: Attachment


class AttachmentDeleteResponse(BaseModel):
    success: bool


class AttachmentGetDataResponse(BaseModel):
    data: BinaryIO

    model_config = ConfigDict(arbitrary_types_allowed=True)


class AttachmentGetThumbnailResponse(BaseModel):
    url: str



from typing import List

from pydantic import BaseModel

from ..pagination import Pagination
from .alert import UserAlert

__all__ = (
    "AlertGetResponse",
    "AlertMarkResponse",
    "AlertSendResponse",
    "AlertsGetResponse",
    "AlertsMarkAllResponse",
)


class AlertsGetResponse(BaseModel):
    alerts: List[UserAlert]
    pagination: Pagination


class AlertSendResponse(BaseModel):
    success: bool


class AlertsMarkAllResponse(BaseModel):
    success: bool


class AlertGetResponse(BaseModel):
    alert: UserAlert


class AlertMarkResponse(BaseModel):
    success: bool



"""XenForo API endpoint definitions."""

from typing import Final

from .endpoint import HTTPMethod, create_endpoint

ENDPOINT_API: Final[str] = "https://forum.arzguard.com/api"

# ============================================================================
# ALERTS
# ============================================================================

endpoint_alerts = create_endpoint(
    ENDPOINT_API + "/alerts",
    HTTPMethod.GET,
    HTTPMethod.POST,
)

endpoint_alerts_mark_all = create_endpoint(
    endpoint_alerts + "/mark-all",
    HTTPMethod.POST,
)


def endpoint_alert(alert_id: int):
    return create_endpoint(
        endpoint_alerts + f"/{alert_id}",
        HTTPMethod.GET,
    )


def endpoint_alert_mark(alert_id: int):
    return create_endpoint(
        endpoint_alert(alert_id) + "/mark",
        HTTPMethod.POST,
    )


# ============================================================================
# ATTACHMENTS
# ============================================================================

endpoint_attachments = create_endpoint(
    ENDPOINT_API + "/attachments",
    HTTPMethod.GET,
    HTTPMethod.POST,
)

endpoint_attachments_new_key = create_endpoint(
    endpoint_attachments + "/new-key",
    HTTPMethod.POST,
)


def endpoint_attachment(attachment_id: int):
    return create_endpoint(
        endpoint_attachments + f"/{attachment_id}",
        HTTPMethod.GET,
        HTTPMethod.DELETE,
    )


def endpoint_attachment_data(attachment_id: int):
    return create_endpoint(
        endpoint_attachment(attachment_id) + "/data",
        HTTPMethod.GET,
    )


def endpoint_attachment_thumbnail(attachment_id: int):
    return create_endpoint(
        endpoint_attachment(attachment_id) + "/thumbnail",
        HTTPMethod.GET,
    )


# ============================================================================
# AUTH
# ============================================================================

endpoint_auth = create_endpoint(
    ENDPOINT_API + "/auth",
    HTTPMethod.POST,
)

endpoint_auth_from_session = create_endpoint(
    endpoint_auth + "/from-session",
    HTTPMethod.POST,
)

endpoint_auth_login_token = create_endpoint(
    endpoint_auth + "/login-token",
    HTTPMethod.POST,
)


# ============================================================================
# CONVERSATION MESSAGES
# ============================================================================

endpoint_conversation_messages = create_endpoint(
    ENDPOINT_API + "/conversation-messages",
    HTTPMethod.POST,
)


def endpoint_conversation_message(message_id: int):
    return create_endpoint(
        endpoint_conversation_messages + f"/{message_id}",
        HTTPMethod.GET,
        HTTPMethod.POST,
    )


def endpoint_conversation_message_react(message_id: int):
    return create_endpoint(
        endpoint_conversation_message(message_id) + "/react",
        HTTPMethod.POST,
    )


# ============================================================================
# CONVERSATIONS
# ============================================================================

endpoint_conversations = create_endpoint(
    ENDPOINT_API + "/conversations",
    HTTPMethod.GET,
    HTTPMethod.POST,
)


def endpoint_conversation(conversation_id: int):
    return create_endpoint(
        endpoint_conversations + f"/{conversation_id}",
        HTTPMethod.GET,
        HTTPMethod.POST,
        HTTPMethod.DELETE,
    )


def endpoint_conversation_invite(conversation_id: int):
    return create_endpoint(
        endpoint_conversation(conversation_id) + "/invite",
        HTTPMethod.POST,
    )


def endpoint_conversation_mark_read(conversation_id: int):
    return create_endpoint(
        endpoint_conversation(conversation_id) + "/mark-read",
        HTTPMethod.POST,
    )


def endpoint_conversation_mark_unread(conversation_id: int):
    return create_endpoint(
        endpoint_conversation(conversation_id) + "/mark-unread",
        HTTPMethod.POST,
    )


def endpoint_conversation_messages_list(conversation_id: int):
    return create_endpoint(
        endpoint_conversation(conversation_id) + "/messages",
        HTTPMethod.GET,
    )


def endpoint_conversation_star(conversation_id: int):
    return create_endpoint(
        endpoint_conversation(conversation_id) + "/star",
        HTTPMethod.POST,
    )


# ============================================================================
# FORUMS
# ============================================================================


def endpoint_forum(forum_id: int):
    return create_endpoint(
        ENDPOINT_API + f"/forums/{forum_id}",
        HTTPMethod.GET,
    )


def endpoint_forum_mark_read(forum_id: int):
    return create_endpoint(
        endpoint_forum(forum_id) + "/mark-read",
        HTTPMethod.POST,
    )


def endpoint_forum_threads(forum_id: int):
    return create_endpoint(
        endpoint_forum(forum_id) + "/threads",
        HTTPMethod.GET,
    )


# ============================================================================
# INDEX
# ============================================================================

endpoint_index = create_endpoint(
    ENDPOINT_API + "/index",
    HTTPMethod.GET,
)


# ============================================================================
# ME (Current User)
# ============================================================================

endpoint_me = create_endpoint(
    ENDPOINT_API + "/me",
    HTTPMethod.GET,
    HTTPMethod.POST,
)

endpoint_me_avatar = create_endpoint(
    endpoint_me + "/avatar",
    HTTPMethod.POST,
    HTTPMethod.DELETE,
)

endpoint_me_email = create_endpoint(
    endpoint_me + "/email",
    HTTPMethod.POST,
)

endpoint_me_password = create_endpoint(
    endpoint_me + "/password",
    HTTPMethod.POST,
)


# ============================================================================
# NODES
# ============================================================================

endpoint_nodes = create_endpoint(
    ENDPOINT_API + "/nodes",
    HTTPMethod.GET,
    HTTPMethod.POST,
)

endpoint_nodes_flattened = create_endpoint(
    endpoint_nodes + "/flattened",
    HTTPMethod.GET,
)


def endpoint_node(node_id: int):
    return create_endpoint(
        endpoint_nodes + f"/{node_id}",
        HTTPMethod.GET,
        HTTPMethod.POST,
        HTTPMethod.DELETE,
    )


# ============================================================================
# POSTS
# ============================================================================

endpoint_posts = create_endpoint(
    ENDPOINT_API + "/posts",
    HTTPMethod.POST,
)


def endpoint_post(post_id: int):
    return create_endpoint(
        endpoint_posts + f"/{post_id}",
        HTTPMethod.GET,
        HTTPMethod.POST,
        HTTPMethod.DELETE,
    )


def endpoint_post_mark_solution(post_id: int):
    return create_endpoint(
        endpoint_post(post_id) + "/mark-solution",
        HTTPMethod.POST,
    )


def endpoint_post_react(post_id: int):
    return create_endpoint(
        endpoint_post(post_id) + "/react",
        HTTPMethod.POST,
    )


def endpoint_post_vote(post_id: int):
    return create_endpoint(
        endpoint_post(post_id) + "/vote",
        HTTPMethod.POST,
    )


# ============================================================================
# PROFILE POST COMMENTS
# ============================================================================

endpoint_profile_post_comments = create_endpoint(
    ENDPOINT_API + "/profile-post-comments",
    HTTPMethod.POST,
)


def endpoint_profile_post_comment(comment_id: int):
    return create_endpoint(
        endpoint_profile_post_comments + f"/{comment_id}",
        HTTPMethod.GET,
        HTTPMethod.POST,
        HTTPMethod.DELETE,
    )


def endpoint_profile_post_comment_react(comment_id: int):
    return create_endpoint(
        endpoint_profile_post_comment(comment_id) + "/react",
        HTTPMethod.POST,
    )


# ============================================================================
# PROFILE POSTS
# ============================================================================

endpoint_profile_posts = create_endpoint(
    ENDPOINT_API + "/profile-posts",
    HTTPMethod.POST,
)


def endpoint_profile_post(profile_post_id: int):
    return create_endpoint(
        endpoint_profile_posts + f"/{profile_post_id}",
        HTTPMethod.GET,
        HTTPMethod.POST,
        HTTPMethod.DELETE,
    )


def endpoint_profile_post_comments_list(profile_post_id: int):
    return create_endpoint(
        endpoint_profile_post(profile_post_id) + "/comments",
        HTTPMethod.GET,
    )


def endpoint_profile_post_react(profile_post_id: int):
    return create_endpoint(
        endpoint_profile_post(profile_post_id) + "/react",
        HTTPMethod.POST,
    )


# ============================================================================
# STATS
# ============================================================================

endpoint_stats = create_endpoint(
    ENDPOINT_API + "/stats",
    HTTPMethod.GET,
)


# ============================================================================
# THREADS
# ============================================================================

endpoint_threads = create_endpoint(
    ENDPOINT_API + "/threads",
    HTTPMethod.GET,
    HTTPMethod.POST,
)


def endpoint_thread(thread_id: int):
    return create_endpoint(
        endpoint_threads + f"/{thread_id}",
        HTTPMethod.GET,
        HTTPMethod.POST,
        HTTPMethod.DELETE,
    )


def endpoint_thread_change_type(thread_id: int):
    return create_endpoint(
        endpoint_thread(thread_id) + "/change-type",
        HTTPMethod.POST,
    )


def endpoint_thread_mark_read(thread_id: int):
    return create_endpoint(
        endpoint_thread(thread_id) + "/mark-read",
        HTTPMethod.POST,
    )


def endpoint_thread_move(thread_id: int):
    return create_endpoint(
        endpoint_thread(thread_id) + "/move",
        HTTPMethod.POST,
    )


def endpoint_thread_posts(thread_id: int):
    return create_endpoint(
        endpoint_thread(thread_id) + "/posts",
        HTTPMethod.GET,
    )


def endpoint_thread_vote(thread_id: int):
    return create_endpoint(
        endpoint_thread(thread_id) + "/vote",
        HTTPMethod.POST,
    )


# ============================================================================
# USERS
# ============================================================================

endpoint_users = create_endpoint(
    ENDPOINT_API + "/users",
    HTTPMethod.GET,
    HTTPMethod.POST,
)

endpoint_users_find_email = create_endpoint(
    endpoint_users + "/find-email",
    HTTPMethod.GET,
)

endpoint_users_find_name = create_endpoint(
    endpoint_users + "/find-name",
    HTTPMethod.GET,
)


def endpoint_user(user_id: int):
    return create_endpoint(
        endpoint_users + f"/{user_id}",
        HTTPMethod.GET,
        HTTPMethod.POST,
        HTTPMethod.DELETE,
    )


def endpoint_user_avatar(user_id: int):
    return create_endpoint(
        endpoint_user(user_id) + "/avatar",
        HTTPMethod.POST,
        HTTPMethod.DELETE,
    )


def endpoint_user_profile_posts(user_id: int):
    return create_endpoint(
        endpoint_user(user_id) + "/profile-posts",
        HTTPMethod.GET,
    )


# ============================================================================
# ACTIONS
# ============================================================================


endpoint_demote = create_endpoint(
    ENDPOINT_API + "/demote",
    HTTPMethod.GET,
)


def endpoint_demote_user(user_id: int):
    return create_endpoint(
        endpoint_demote + f"/{user_id}",
        HTTPMethod.POST,
    )


endpoint_promote = create_endpoint(
    ENDPOINT_API + "/promote",
    HTTPMethod.GET,
)


def endpoint_promote_user(user_id: int):
    return create_endpoint(
        endpoint_promote + f"/{user_id}",
        HTTPMethod.POST,
    )



from typing import List, Optional

from pydantic import BaseModel, field_serializer

__all__ = (
    "ConversationCreateParams",
    "ConversationDeleteParams",
    "ConversationGetMessagesParams",
    "ConversationGetParams",
    "ConversationInviteParams",
    "ConversationMarkReadParams",
    "ConversationStarParams",
    "ConversationUpdateParams",
    "ConversationsGetParams",
)


class ConversationsGetParams(BaseModel):
    page: Optional[int] = None
    starter_id: Optional[int] = None
    receiver_id: Optional[int] = None
    starred: Optional[bool] = None
    unread: Optional[bool] = None

    @field_serializer("starred", "unread")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ConversationCreateParams(BaseModel):
    recipient_ids: List[int]
    title: str
    message: str
    attachment_key: Optional[str] = None
    conversation_open: Optional[bool] = None
    open_invite: Optional[bool] = None

    @field_serializer("open_invite", "conversation_open")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ConversationGetParams(BaseModel):
    with_messages: Optional[bool] = None
    page: Optional[int] = None

    @field_serializer("with_messages")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ConversationUpdateParams(BaseModel):
    title: Optional[str] = None
    open_invite: Optional[bool] = None
    conversation_open: Optional[bool] = None

    @field_serializer("open_invite", "conversation_open")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ConversationDeleteParams(BaseModel):
    ignore: bool

    @field_serializer("ignore")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ConversationInviteParams(BaseModel):
    recipient_ids: List[int]


class ConversationMarkReadParams(BaseModel):
    date: int


class ConversationGetMessagesParams(BaseModel):
    page: int


class ConversationStarParams(BaseModel):
    star: bool

    @field_serializer("star")
    def serialize_bool(self, v: bool):
        return 1 if v else 0



from typing import Any, Dict, List, Union

from pydantic import BaseModel, field_validator

__all__ = (
    "ForumTypeData",
    "SearchForumTypeData",
)


class ForumTypeData(BaseModel):
    allow_posting: bool
    can_create_thread: bool
    can_upload_attachment: bool
    custom_fields: Dict[str, Any]

    @field_validator("custom_fields", mode="before")
    @classmethod
    def validate_scopes(
        cls, v: Union[Dict[str, Any], List[Any]]
    ) -> Dict[str, Any]:
        if isinstance(v, list):
            v = {}

        return v


class SearchForumTypeData(BaseModel):
    discussion_count: int
    message_count: int


class LinkForumTypeData(BaseModel):
    link_url: str
    redirect_count: int



from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, field_validator

from ...types.api_key_type import ApiKeyTypeEnum
from ...types.api_scopes import APIScopeIdsEnum

__all__ = ("ApiKey", "IndexGetResponse")


class ApiKey(BaseModel):
    type: ApiKeyTypeEnum
    user_id: Optional[int] = None
    allow_all_scopes: bool
    scopes: Dict[APIScopeIdsEnum, bool]

    @field_validator("scopes", mode="before")
    @classmethod
    def validate_scopes(
        cls, v: Union[Dict[APIScopeIdsEnum, bool], List[Any]]
    ) -> Dict[APIScopeIdsEnum, bool]:
        if isinstance(v, list):
            v = {}

        return v


class IndexGetResponse(BaseModel):
    version_id: int
    site_title: str
    base_url: str
    api_url: str
    key: ApiKey



from enum import Enum

__all__ = ("ArzGuardGroupsIdsEnum", "ArzGuardGroupsNamesEnum")


class ArzGuardGroupsIdsEnum(Enum):
    FORUM_MANAGERS = 3
    FORUM_ADMINISTRATORS = 25
    FORUM_MODERATORS = 36
    SERVICE_ACCOUNTS = 19
    GAME_ADMINISTRATORS = 17
    SERVER_MANAGERS = 33
    HEAD_MODERATORS = 4
    DEPUTY_HEAD_MODERATORS = 9
    TECH_MODERATORS = 32
    CURATORS_OF_MODERATION = 20
    SENIOR_MODERATORS = 7
    MODERATORS = 8
    BLOCKED_USERS = 5


class ArzGuardGroupsNamesEnum(Enum):
    FORUM_MANAGERS = "(01) Руководители форума"
    FORUM_ADMINISTRATORS = "(02) Администраторы форума"
    FORUM_MODERATORS = "(03) Модераторы форума"
    SERVICE_ACCOUNTS = "(04) Служебные аккаунты"
    GAME_ADMINISTRATORS = "(05) Игровые администраторы"
    SERVER_MANAGERS = "(06) Руководители серверов"
    HEAD_MODERATORS = "(07) Главные модераторы"
    DEPUTY_HEAD_MODERATORS = "(08) Заместители главных модераторов"
    TECH_MODERATORS = "(09) Технические модераторы"
    CURATORS_OF_MODERATION = "(10) Кураторы модерации"
    SENIOR_MODERATORS = "(11) Старшие модераторы"
    MODERATORS = "(12) Модераторы"
    USERS = "(13) Пользователи"
    GUESTS = "(14) Гости"
    BLOCKED_USERS = "(15) Заблокированные пользователи"



from typing import Dict, Optional

from pydantic import BaseModel, Field

from ..user import User

__all__ = ("Conversation",)


class Conversation(BaseModel):
    username: str  # Name of the user that started the conversation
    recipients: Dict[
        str, str
    ]  # Key-value pair of recipient user IDs and names
    is_starred: bool  # True if the viewing user starred the conversation
    is_unread: Optional[bool] = Field(default=None)
    can_edit: bool
    can_reply: bool
    can_invite: bool
    can_upload_attachment: bool
    view_url: str
    conversation_id: int
    title: str
    user_id: int
    start_date: int
    open_invite: bool
    conversation_open: bool
    reply_count: int
    recipient_count: int
    first_message_id: int
    last_message_date: int
    last_message_id: int
    last_message_user_id: int
    starter: Optional[User] = Field(alias="Starter", default=None)



from pydantic import BaseModel

from .stats import LatestUser, Online, Totals

__all__ = ("StatsResponse",)


class StatsResponse(BaseModel):
    totals: Totals
    latest_user: LatestUser
    online: Online



from enum import Enum
from typing import List, Optional

from pydantic import BaseModel, Field

from ..attachment import Attachment
from ..conversation import Conversation
from ..user import User

__all__ = ("ConversationMessage", "ConversationMessageReactActionEnum")


class ConversationMessageReactActionEnum(Enum):
    INSERT = "insert"
    DELETE = "delete"


class ConversationMessage(BaseModel):
    username: str
    is_unread: (
        Optional[
            bool
        ]  #  If accessing as a user, true if this conversation message is unread
    )
    message_parsed: str  # HTML parsed version of the message contents.
    can_edit: bool
    can_react: bool
    view_url: str
    conversation: Optional[Conversation] = Field(
        default=None, alias="Conversation"
    )  # If requested by context, the conversation this message is part of.
    attachments: Optional[List[Attachment]] = Field(
        default=None, alias="Attachments"
    )
    is_reacted_to: bool  # True if the viewing user has reacted to this content
    visitor_reaction_id: Optional[int] = None
    message_id: int
    conversation_id: int
    message_date: int
    user_id: int
    message: str
    attach_count: int
    reaction_score: int
    user: Optional[User] = Field(default=None, alias="User")



from typing import List, Optional

from pydantic import BaseModel

from ...types.post_react_state import PostReactStateEnum
from ..pagination import Pagination
from ..post import Post
from .thread import Thread

__all__ = (
    "ThreadChangeTypeResponse",
    "ThreadCreateResponse",
    "ThreadDeleteResponse",
    "ThreadGetResponse",
    "ThreadMarkReadResponse",
    "ThreadMoveResponse",
    "ThreadPostsGetResponse",
    "ThreadUpdateResponse",
    "ThreadVoteResponse",
    "ThreadsGetResponse",
)


class ThreadsGetResponse(BaseModel):
    threads: List[Thread]
    pagination: Pagination


class ThreadGetResponse(BaseModel):
    thread: Thread
    first_unread: Optional[Post] = None
    first_post: Optional[Post] = None
    last_post: Optional[Post] = None
    pinned_post: Optional[Post] = None
    highlighted_posts: Optional[List[Post]] = None
    posts: Optional[List[Post]] = None
    pagination: Optional[Pagination] = None


class ThreadCreateResponse(BaseModel):
    success: bool
    thread: Thread


class ThreadUpdateResponse(BaseModel):
    success: bool
    thread: Thread


class ThreadDeleteResponse(BaseModel):
    success: bool


class ThreadChangeTypeResponse(BaseModel):
    success: bool
    thread: Thread


class ThreadMarkReadResponse(BaseModel):
    success: bool


class ThreadMoveResponse(BaseModel):
    success: bool
    thread: Thread


class ThreadPostsGetResponse(BaseModel):
    pinned_post: Optional[Post] = None
    highlighted_posts: Optional[List[Post]] = None
    posts: List[Post]
    pagination: Pagination


class ThreadVoteResponse(BaseModel):
    success: bool
    action: PostReactStateEnum



from typing import TYPE_CHECKING, List, Optional

from pydantic import BaseModel, Field

from ..attachment import Attachment

if TYPE_CHECKING:
    from ..profile_post import ProfilePost
    from ..user import User


__all__ = ("ProfilePostComment",)


class ProfilePostComment(BaseModel):
    username: str
    message_parsed: str  # HTML parsed version of the message contents.
    can_edit: bool
    can_soft_delete: bool
    can_hard_delete: bool
    can_react: bool
    can_view_attachments: bool
    Attachments: Optional[List[Attachment]] = Field(
        alias="Attachments", default=None
    )  # 	 Attachments to this profile post, if it has any.
    ProfilePost: Optional[
        "ProfilePost"
    ]  #  If requested by context, the profile post this comment relates to.
    is_reacted_to: bool  # True if the viewing user has reacted to this content
    visitor_reaction_id: Optional[int] = None
    profile_post_comment_id: int
    profile_post_id: int
    user_id: int
    comment_date: int
    message: str
    message_state: str
    warning_message: str
    reaction_score: int
    User: "User"



from enum import Enum
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, field_serializer

from ...types.direction import DirectionTypeEnum
from ...types.thread_type import ThreadTypeEnum
from ..vote_type import VoteTypeEnum

__all__ = (
    "OrderField",
    "ThreadChangeTypeParams",
    "ThreadCreateParams",
    "ThreadDeleteParams",
    "ThreadGetParams",
    "ThreadMarkReadParams",
    "ThreadMoveParams",
    "ThreadPostsGetParams",
    "ThreadUpdateParams",
    "ThreadVoteParams",
    "ThreadsGetParams",
)


class OrderField(Enum):
    LAST_POST_DATE = "last_post_date"
    POST_DATE = "post_date"
    TITLE = "title"
    REPLY_COUNT = "reply_count"
    VIEW_COUNT = "view_count"
    VOTE_SCORE = "vote_score"
    FIRST_POST_REACTION_SCORE = "first_post_reaction_score"


class ThreadsGetParams(BaseModel):
    page: Optional[int] = None
    prefix_id: Optional[int] = None
    starter_id: Optional[int] = None
    last_days: Optional[int] = None
    unread: Optional[bool] = None
    thread_type: Optional[ThreadTypeEnum] = None
    order: Optional[OrderField] = None
    direction: Optional[DirectionTypeEnum] = None

    @field_serializer("unread")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ThreadCreateParams(BaseModel):
    node_id: int
    title: str
    message: str
    discussion_type: Optional[str] = None
    prefix_id: Optional[int] = None
    tags: Optional[List[str]] = None
    custom_fields: Optional[Dict[str, Any]] = None
    discussion_open: Optional[bool] = None
    sticky: Optional[bool] = None
    attachment_key: Optional[str] = None

    @field_serializer("discussion_open", "sticky")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ThreadGetParams(BaseModel):
    with_posts: Optional[bool] = None
    page: Optional[int] = None
    with_first_post: Optional[bool] = None
    with_last_post: Optional[bool] = None
    order: Optional[str] = None

    @field_serializer("with_posts", "with_last_post", "with_first_post")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ThreadUpdateParams(BaseModel):
    prefix_id: Optional[int] = None
    title: Optional[str] = None
    discussion_open: Optional[bool] = None
    sticky: Optional[bool] = None
    custom_fields: Optional[Dict[str, str]] = None
    add_tags: Optional[List[str]] = None
    remove_tags: Optional[List[str]] = None

    @field_serializer("discussion_open", "sticky")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ThreadDeleteParams(BaseModel):
    hard_delete: Optional[bool] = None
    reason: Optional[str] = None
    starter_alert: Optional[bool] = None
    starter_alert_reason: Optional[str] = None

    @field_serializer("hard_delete", "starter_alert")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ThreadChangeTypeParams(BaseModel):
    new_thread_type_id: ThreadTypeEnum

    @field_serializer("new_thread_type_id")
    def serialize_type(self, v: ThreadTypeEnum):
        return v.value


class ThreadMarkReadParams(BaseModel):
    date: int


class ThreadMoveParams(BaseModel):
    target_node_id: int
    prefix_id: Optional[int] = None
    title: Optional[str] = None
    notify_watchers: Optional[bool] = None
    starter_alert: Optional[bool] = None
    starter_alert_reason: Optional[str] = None

    @field_serializer("notify_watchers", "starter_alert")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class ThreadPostsGetParams(BaseModel):
    page: Optional[int] = None
    order: Optional[str] = None


class ThreadVoteParams(BaseModel):
    type: VoteTypeEnum

    @field_serializer("type")
    def serialize_type(self, v: ThreadTypeEnum):
        return v.value



from typing import List, Optional

from pydantic import BaseModel

from ...types.post_react_state import PostReactStateEnum
from ..pagination import Pagination
from ..profile_post_comment import ProfilePostComment
from .profile_post import ProfilePost

__all__ = (
    "ProfilePostCommentsGetResponse",
    "ProfilePostCreateResponse",
    "ProfilePostDeleteResponse",
    "ProfilePostGetResponse",
    "ProfilePostReactResponse",
    "ProfilePostUpdateResponse",
)


class ProfilePostCreateResponse(BaseModel):
    success: bool
    profile_post: ProfilePost


class ProfilePostGetResponse(BaseModel):
    profile_post: ProfilePost
    comments: Optional[List[ProfilePostComment]] = None
    pagination: Optional[Pagination] = None


class ProfilePostUpdateResponse(BaseModel):
    success: bool
    profile_post: ProfilePost


class ProfilePostDeleteResponse(BaseModel):
    success: bool


class ProfilePostCommentsGetResponse(BaseModel):
    comments: List[ProfilePostComment]
    pagination: Pagination


class ProfilePostReactResponse(BaseModel):
    success: bool
    action: PostReactStateEnum



from typing import TYPE_CHECKING, List, Optional

from pydantic import BaseModel, Field

from ..attachment import Attachment

if TYPE_CHECKING:
    from ..profile_post_comment import ProfilePostComment
    from ..user import User

__all__ = ("ProfilePost",)


class ProfilePost(BaseModel):
    username: str
    message_parsed: str  # HTML parsed version of the message contents.
    can_edit: bool
    can_soft_delete: bool
    can_hard_delete: bool
    can_react: bool
    can_view_attachments: bool
    view_url: str
    profile_user: (
        Optional[
            "User"
        ]  # If requested by context, the user this profile post was left for.
    ) = Field(alias="ProfileUser", default=None)
    Attachments: Optional[List[Attachment]] = Field(
        alias="Attachments", default=None
    )  # 	 Attachments to this profile post, if it has any.
    latest_comments: Optional[List["ProfilePostComment"]] = Field(
        alias="LatestComments", default=None
    )  # If requested, the most recent comments on this profile post.
    is_reacted_to: bool  # True if the viewing user has reacted to this content
    visitor_reaction_id: Optional[int] = None
    profile_post_id: int
    profile_user_id: int
    user_id: int
    post_date: int
    message: str
    message_state: str
    warning_message: str
    comment_count: int
    first_comment_date: int
    last_comment_date: int
    reaction_score: int
    user: Optional["User"] = Field(alias="User", default=None)



from typing import Optional

from pydantic import BaseModel, field_serializer

from ..vote_type import VoteTypeEnum

__all__ = (
    "PostCreateParams",
    "PostDeleteParams",
    "PostReactParams",
    "PostUpdateParams",
    "PostVoteParams",
)


class PostCreateParams(BaseModel):
    thread_id: int
    message: str
    attachment_key: Optional[str] = None


class PostUpdateParams(BaseModel):
    message: str
    silent: Optional[bool] = None
    clear_edit: Optional[bool] = None
    author_alert: Optional[bool] = None
    author_alert_reason: Optional[str] = None
    attachment_key: Optional[str] = None

    @field_serializer("silent", "clear_edit", "author_alert")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class PostDeleteParams(BaseModel):
    hard_delete: Optional[bool] = None
    reason: Optional[str] = None
    author_alert: Optional[bool] = None
    author_alert_reason: Optional[str] = None

    @field_serializer("hard_delete", "author_alert")
    def serialize_bool(self, v: bool):
        return 1 if v else 0


class PostReactParams(BaseModel):
    reaction_id: int


class PostVoteParams(BaseModel):
    type: VoteTypeEnum

    @field_serializer("type")
    def serialize_type(self, v: VoteTypeEnum):
        return v.value



from pydantic import BaseModel

from .conversation_message import ConversationMessage

__all__ = (
    "ConversationMessageGetResponse",
    "ConversationMessageReactResponse",
    "ConversationMessageReplyResponse",
    "ConversationMessageUpdateResponse",
)


class ConversationMessageReplyResponse(BaseModel):
    success: bool
    message: ConversationMessage


class ConversationMessageUpdateResponse(BaseModel):
    success: bool
    message: ConversationMessage


class ConversationMessageReactResponse(BaseModel):
    success: bool
    action: str


class ConversationMessageGetResponse(BaseModel):
    message: ConversationMessage



from enum import Enum

from pydantic import BaseModel

from ...types.content_type import ContentTypeEnum
from ..user import User

__all__ = ("AlertActionTypeEnum", "UserAlert")


class AlertActionTypeEnum(Enum):
    INSERT = "insert"
    AWARD = "award"
    TROPHY = "trophy"


class UserAlert(BaseModel):
    action: AlertActionTypeEnum
    alert_id: int
    alert_text: str
    alert_url: str
    alerted_user_id: int
    auto_read: bool
    content_id: int
    content_type: ContentTypeEnum
    event_date: int
    read_date: int
    User: User
    user_id: int
    username: str
    view_date: int



"""Custom exceptions for NightForo."""


class NightForoBaseError(Exception):
    """Base exception for NightForo."""


class XenForoError(Exception):
    """Exception raised for XenForo API errors."""

    def __init__(self, msg: object) -> None:
        super().__init__(msg)


class UnsupportedEndpointMethodError(XenForoError):
    """Exception raised when an unsupported HTTP method is used."""


class NoApiKeyProvidedError(NightForoBaseError):
    """Exception raised when no API key is provided."""



"""HTTP client for XenForo API."""

from __future__ import annotations

import logging
from typing import TYPE_CHECKING, BinaryIO

import aiohttp

from . import __version__
from .endpoint import HTTPMethod
from .endpoints import (
    endpoint_alert,
    endpoint_alert_mark,
    endpoint_alerts,
    endpoint_alerts_mark_all,
    endpoint_attachment,
    endpoint_attachment_data,
    endpoint_attachment_thumbnail,
    endpoint_attachments,
    endpoint_attachments_new_key,
    endpoint_auth,
    endpoint_auth_from_session,
    endpoint_auth_login_token,
    endpoint_conversation,
    endpoint_conversation_invite,
    endpoint_conversation_mark_read,
    endpoint_conversation_mark_unread,
    endpoint_conversation_message,
    endpoint_conversation_message_react,
    endpoint_conversation_messages,
    endpoint_conversation_messages_list,
    endpoint_conversation_star,
    endpoint_conversations,
    endpoint_demote,
    endpoint_demote_user,
    endpoint_forum,
    endpoint_forum_mark_read,
    endpoint_forum_threads,
    endpoint_index,
    endpoint_me,
    endpoint_me_avatar,
    endpoint_me_email,
    endpoint_me_password,
    endpoint_node,
    endpoint_nodes,
    endpoint_nodes_flattened,
    endpoint_post,
    endpoint_post_mark_solution,
    endpoint_post_react,
    endpoint_post_vote,
    endpoint_posts,
    endpoint_profile_post,
    endpoint_profile_post_comment,
    endpoint_profile_post_comment_react,
    endpoint_profile_post_comments,
    endpoint_profile_post_comments_list,
    endpoint_profile_post_react,
    endpoint_profile_posts,
    endpoint_promote,
    endpoint_promote_user,
    endpoint_stats,
    endpoint_thread,
    endpoint_thread_change_type,
    endpoint_thread_mark_read,
    endpoint_thread_move,
    endpoint_thread_posts,
    endpoint_thread_vote,
    endpoint_threads,
    endpoint_user,
    endpoint_user_avatar,
    endpoint_user_profile_posts,
    endpoint_users,
    endpoint_users_find_email,
    endpoint_users_find_name,
)
from .errors import UnsupportedEndpointMethodError, XenForoError
from .types.file import XenforoFile

if TYPE_CHECKING:
    from typing import Any

    from pydantic import BaseModel

    from .endpoint import Endpoint
    from .types.alert.params import (
        AlertMarkParams,
        AlertSendParams,
        AlertsGetParams,
        AlertsMarkAllParams,
    )
    from .types.attachment.params import (
        AttachmentsCreateNewKeyParams,
        AttachmentsGetParams,
        AttachmentUploadParams,
    )
    from .types.auth.params import (
        AuthFromSessionParams,
        AuthLoginTokenParams,
        AuthTestParams,
    )
    from .types.conversation.params import (
        ConversationCreateParams,
        ConversationDeleteParams,
        ConversationGetMessagesParams,
        ConversationGetParams,
        ConversationInviteParams,
        ConversationMarkReadParams,
        ConversationsGetParams,
        ConversationStarParams,
        ConversationUpdateParams,
    )
    from .types.conversation_message.params import (
        ConversationMessageReactParams,
        ConversationMessageReplyParams,
        ConversationMessageUpdateParams,
    )
    from .types.forum.params import (
        ForumGetParams,
        ForumMarkReadParams,
        ForumThreadsGetParams,
    )
    from .types.me.params import (
        MeEmailUpdateParams,
        MePasswordUpdateParams,
        MeUpdateParams,
    )
    from .types.node.params import (
        AnyNodeCreateParams,
        NodeDeleteParams,
        NodeUpdateParams,
    )
    from .types.post.params import (
        PostCreateParams,
        PostDeleteParams,
        PostReactParams,
        PostUpdateParams,
        PostVoteParams,
    )
    from .types.profile_post.params import (
        ProfilePostCreateParams,
        ProfilePostDeleteParams,
        ProfilePostGetParams,
        ProfilePostReactParams,
        ProfilePostUpdateParams,
    )
    from .types.profile_post_comment.params import (
        ProfilePostCommentCreateParams,
        ProfilePostCommentDeleteParams,
        ProfilePostCommentReactParams,
        ProfilePostCommentsGetParams,
        ProfilePostCommentUpdateParams,
    )
    from .types.thread.params import (
        ThreadChangeTypeParams,
        ThreadCreateParams,
        ThreadDeleteParams,
        ThreadGetParams,
        ThreadMarkReadParams,
        ThreadMoveParams,
        ThreadPostsGetParams,
        ThreadsGetParams,
        ThreadUpdateParams,
        ThreadVoteParams,
    )
    from .types.user.params import (
        UserCreateParams,
        UserDeleteParams,
        UserDemoteParams,
        UserGetParams,
        UserProfilePostsGetParams,
        UserPromoteParams,
        UsersFindEmailParams,
        UsersFindNameParams,
        UsersGetParams,
        UserUpdateParams,
    )

_log = logging.getLogger(__name__)


class HTTPClient:
    def __init__(
        self,
        api_key: str,
        is_super_user: bool = False,
        xf_user_id: int | None = None,
    ) -> None:
        self.api_key = api_key
        self.xf_user_id = xf_user_id
        self.is_super_user = is_super_user

    async def _request(
        self,
        endpoint: Endpoint,
        method: HTTPMethod,
        body_params: BaseModel | None = None,
        query_params: BaseModel | None = None,
        file: XenforoFile | None = None,
    ) -> Any:
        if method not in endpoint.supported_methods:
            raise UnsupportedEndpointMethodError(method)

        headers: dict[str, str] = {}

        headers["XF-Api-Key"] = self.api_key

        if self.is_super_user:
            headers["XF-Api-User"] = str(self.xf_user_id)

        headers["User-Agent"] = "Nightforo/" + __version__

        data = None
        query = None

        if body_params is not None:
            dump = body_params.model_dump(by_alias=True, exclude_none=True)
            data = aiohttp.FormData(dump)

        if file is not None:
            if data is None:
                data = aiohttp.FormData()

            data.add_field(file.name, file.stream)

        if query_params:
            query = query_params.model_dump(by_alias=True, exclude_none=True)

        async with aiohttp.ClientSession() as session, session.request(
            method=method.value,
            url=endpoint.url,
            data=data,
            headers=headers,
            params=query,
        ) as response:
            _log.debug(
                "%s %s with query=%s has returned %s",
                method,
                endpoint.url,
                query,
                response.status,
            )

            try:
                payload = await response.json()
            except aiohttp.ContentTypeError:
                raise XenForoError(  # noqa: B904
                    f"Response is not JSON. Status: {response.status}"
                )

            if (errors := payload.get("errors", None)) is not None:
                raise XenForoError(errors)

            if (errors := payload.get("error", None)) is not None:
                raise XenForoError(errors)

            return payload

    # ============================================================================
    # ALERTS
    # ============================================================================

    async def get_alerts(self, params: AlertsGetParams | None = None) -> Any:
        return await self._request(
            endpoint=endpoint_alerts,
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def send_alert(self, params: AlertSendParams) -> Any:
        return await self._request(
            endpoint=endpoint_alerts,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def mark_all_alerts(self, params: AlertsMarkAllParams) -> Any:
        return await self._request(
            endpoint=endpoint_alerts_mark_all,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_alert(self, alert_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_alert(alert_id), method=HTTPMethod.GET
        )

    async def mark_alert(self, alert_id: int, params: AlertMarkParams) -> Any:
        return await self._request(
            endpoint=endpoint_alert_mark(alert_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    # ============================================================================
    # ATTACHMENTS
    # ============================================================================

    async def get_attachments(self, params: AttachmentsGetParams) -> Any:
        return await self._request(
            endpoint=endpoint_attachments,
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def upload_attachment(
        self, params: AttachmentUploadParams, attachment: BinaryIO
    ) -> Any:
        file = XenforoFile(attachment, "attachment")
        return await self._request(
            endpoint=endpoint_attachments,
            method=HTTPMethod.POST,
            body_params=params,
            file=file,
        )

    async def create_attachment_key(
        self,
        params: AttachmentsCreateNewKeyParams,
        attachment: BinaryIO | None,
    ) -> Any:
        file = XenforoFile(attachment, "attachment") if attachment else None
        return await self._request(
            endpoint=endpoint_attachments_new_key,
            method=HTTPMethod.POST,
            body_params=params,
            file=file,
        )

    async def get_attachment(self, attachment_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_attachment(attachment_id), method=HTTPMethod.GET
        )

    async def delete_attachment(self, attachment_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_attachment(attachment_id),
            method=HTTPMethod.DELETE,
        )

    async def get_attachment_data(self, attachment_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_attachment_data(attachment_id),
            method=HTTPMethod.GET,
        )

    async def get_attachment_thumbnail(self, attachment_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_attachment_thumbnail(attachment_id),
            method=HTTPMethod.GET,
        )

    # ============================================================================
    # AUTH
    # ============================================================================

    async def test_auth(self, params: AuthTestParams) -> Any:
        return await self._request(
            endpoint=endpoint_auth, method=HTTPMethod.POST, body_params=params
        )

    async def auth_from_session(self, params: AuthFromSessionParams) -> Any:
        return await self._request(
            endpoint=endpoint_auth_from_session,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def create_login_token(self, params: AuthLoginTokenParams) -> Any:
        return await self._request(
            endpoint=endpoint_auth_login_token,
            method=HTTPMethod.POST,
            body_params=params,
        )

    # ============================================================================
    # CONVERSATION MESSAGES
    # ============================================================================

    async def reply_conversation_message(
        self, params: ConversationMessageReplyParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation_messages,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_conversation_message(self, message_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_conversation_message(message_id),
            method=HTTPMethod.GET,
        )

    async def update_conversation_message(
        self, message_id: int, params: ConversationMessageUpdateParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation_message(message_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def react_conversation_message(
        self, message_id: int, params: ConversationMessageReactParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation_message_react(message_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    # ============================================================================
    # CONVERSATIONS
    # ============================================================================

    async def get_conversations(
        self, params: ConversationsGetParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversations,
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def create_conversation(
        self, params: ConversationCreateParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversations,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_conversation(
        self,
        conversation_id: int,
        params: ConversationGetParams | None = None,
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation(conversation_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def update_conversation(
        self, conversation_id: int, params: ConversationUpdateParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation(conversation_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def delete_conversation(
        self,
        conversation_id: int,
        params: ConversationDeleteParams | None = None,
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation(conversation_id),
            method=HTTPMethod.DELETE,
            query_params=params,
        )

    async def invite_conversation(
        self, conversation_id: int, params: ConversationInviteParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation_invite(conversation_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def mark_conversation_read(
        self,
        conversation_id: int,
        params: ConversationMarkReadParams | None = None,
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation_mark_read(conversation_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def mark_conversation_unread(self, conversation_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_conversation_mark_unread(conversation_id),
            method=HTTPMethod.POST,
        )

    async def get_conversation_messages(
        self,
        conversation_id: int,
        params: ConversationGetMessagesParams | None = None,
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation_messages_list(conversation_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def star_conversation(
        self, conversation_id: int, params: ConversationStarParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_conversation_star(conversation_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    # ============================================================================
    # FORUMS
    # ============================================================================

    async def get_forum(
        self, forum_id: int, params: ForumGetParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_forum(forum_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def mark_forum_read(
        self, forum_id: int, params: ForumMarkReadParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_forum_mark_read(forum_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_forum_threads(
        self, forum_id: int, params: ForumThreadsGetParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_forum_threads(forum_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    # ============================================================================
    # INDEX
    # ============================================================================

    async def get_index(self) -> Any:
        return await self._request(
            endpoint=endpoint_index, method=HTTPMethod.GET
        )

    # ============================================================================
    # ME (Current User)
    # ============================================================================

    async def get_me(self) -> Any:
        return await self._request(endpoint=endpoint_me, method=HTTPMethod.GET)

    async def update_me(self, params: MeUpdateParams) -> Any:
        return await self._request(
            endpoint=endpoint_me, method=HTTPMethod.POST, body_params=params
        )

    async def update_my_avatar(self, avatar: BinaryIO) -> Any:
        file = XenforoFile(avatar, "avatar")
        return await self._request(
            endpoint=endpoint_me_avatar,
            method=HTTPMethod.POST,
            file=file,
        )

    async def delete_my_avatar(self) -> Any:
        return await self._request(
            endpoint=endpoint_me_avatar, method=HTTPMethod.DELETE
        )

    async def update_my_email(self, params: MeEmailUpdateParams) -> Any:
        return await self._request(
            endpoint=endpoint_me_email,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def update_my_password(self, params: MePasswordUpdateParams) -> Any:
        return await self._request(
            endpoint=endpoint_me_password,
            method=HTTPMethod.POST,
            body_params=params,
        )

    # ============================================================================
    # NODES
    # ============================================================================

    async def get_nodes(self) -> Any:
        return await self._request(
            endpoint=endpoint_nodes, method=HTTPMethod.GET
        )

    async def create_node(self, params: AnyNodeCreateParams) -> Any:
        return await self._request(
            endpoint=endpoint_nodes,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_nodes_flattened(self) -> Any:
        return await self._request(
            endpoint=endpoint_nodes_flattened, method=HTTPMethod.GET
        )

    async def get_node(self, node_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_node(node_id), method=HTTPMethod.GET
        )

    async def update_node(self, node_id: int, params: NodeUpdateParams) -> Any:
        return await self._request(
            endpoint=endpoint_node(node_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def delete_node(
        self, node_id: int, params: NodeDeleteParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_node(node_id),
            method=HTTPMethod.DELETE,
            query_params=params,
        )

    # ============================================================================
    # POSTS
    # ============================================================================

    async def create_post(self, params: PostCreateParams) -> Any:
        return await self._request(
            endpoint=endpoint_posts,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_post(self, post_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_post(post_id), method=HTTPMethod.GET
        )

    async def update_post(self, post_id: int, params: PostUpdateParams) -> Any:
        return await self._request(
            endpoint=endpoint_post(post_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def delete_post(
        self, post_id: int, params: PostDeleteParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_post(post_id),
            method=HTTPMethod.DELETE,
            query_params=params,
        )

    async def mark_post_solution(self, post_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_post_mark_solution(post_id),
            method=HTTPMethod.POST,
        )

    async def react_post(self, post_id: int, params: PostReactParams) -> Any:
        return await self._request(
            endpoint=endpoint_post_react(post_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def vote_post(self, post_id: int, params: PostVoteParams) -> Any:
        return await self._request(
            endpoint=endpoint_post_vote(post_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    # ============================================================================
    # PROFILE POST COMMENTS
    # ============================================================================

    async def create_profile_post_comment(
        self, params: ProfilePostCommentCreateParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post_comments,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_profile_post_comment(self, comment_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post_comment(comment_id),
            method=HTTPMethod.GET,
        )

    async def update_profile_post_comment(
        self, comment_id: int, params: ProfilePostCommentUpdateParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post_comment(comment_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def delete_profile_post_comment(
        self,
        comment_id: int,
        params: ProfilePostCommentDeleteParams | None = None,
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post_comment(comment_id),
            method=HTTPMethod.DELETE,
            query_params=params,
        )

    async def react_profile_post_comment(
        self, comment_id: int, params: ProfilePostCommentReactParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post_comment_react(comment_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    # ============================================================================
    # PROFILE POSTS
    # ============================================================================

    async def create_profile_post(
        self, params: ProfilePostCreateParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_posts,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_profile_post(
        self,
        profile_post_id: int,
        params: ProfilePostGetParams | None = None,
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post(profile_post_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def update_profile_post(
        self, profile_post_id: int, params: ProfilePostUpdateParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post(profile_post_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def delete_profile_post(
        self,
        profile_post_id: int,
        params: ProfilePostDeleteParams | None = None,
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post(profile_post_id),
            method=HTTPMethod.DELETE,
            query_params=params,
        )

    async def get_profile_post_comments(
        self,
        profile_post_id: int,
        params: ProfilePostCommentsGetParams | None = None,
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post_comments_list(profile_post_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def react_profile_post(
        self, profile_post_id: int, params: ProfilePostReactParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_profile_post_react(profile_post_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    # ============================================================================
    # STATS
    # ============================================================================

    async def get_stats(self) -> Any:
        return await self._request(
            endpoint=endpoint_stats, method=HTTPMethod.GET
        )

    # ============================================================================
    # THREADS
    # ============================================================================

    async def get_threads(self, params: ThreadsGetParams | None = None) -> Any:
        return await self._request(
            endpoint=endpoint_threads,
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def create_thread(self, params: ThreadCreateParams) -> Any:
        return await self._request(
            endpoint=endpoint_threads,
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_thread(
        self, thread_id: int, params: ThreadGetParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_thread(thread_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def update_thread(
        self, thread_id: int, params: ThreadUpdateParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_thread(thread_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def delete_thread(
        self, thread_id: int, params: ThreadDeleteParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_thread(thread_id),
            method=HTTPMethod.DELETE,
            query_params=params,
        )

    async def change_thread_type(
        self, thread_id: int, params: ThreadChangeTypeParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_thread_change_type(thread_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def mark_thread_read(
        self, thread_id: int, params: ThreadMarkReadParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_thread_mark_read(thread_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def move_thread(
        self, thread_id: int, params: ThreadMoveParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_thread_move(thread_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_thread_posts(
        self, thread_id: int, params: ThreadPostsGetParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_thread_posts(thread_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def vote_thread(
        self, thread_id: int, params: ThreadVoteParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_thread_vote(thread_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    # ============================================================================
    # USERS
    # ============================================================================

    async def get_users(self, params: UsersGetParams | None = None) -> Any:
        return await self._request(
            endpoint=endpoint_users, method=HTTPMethod.GET, query_params=params
        )

    async def create_user(self, params: UserCreateParams) -> Any:
        return await self._request(
            endpoint=endpoint_users, method=HTTPMethod.POST, body_params=params
        )

    async def find_user_by_email(self, params: UsersFindEmailParams) -> Any:
        return await self._request(
            endpoint=endpoint_users_find_email,
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def find_user_by_name(self, params: UsersFindNameParams) -> Any:
        return await self._request(
            endpoint=endpoint_users_find_name,
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def get_user(
        self, user_id: int, params: UserGetParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_user(user_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    async def update_user(self, user_id: int, params: UserUpdateParams) -> Any:
        return await self._request(
            endpoint=endpoint_user(user_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def delete_user(
        self, user_id: int, params: UserDeleteParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_user(user_id),
            method=HTTPMethod.DELETE,
            query_params=params,
        )

    async def update_user_avatar(self, user_id: int, avatar: BinaryIO) -> Any:
        file = XenforoFile(avatar, "avatar")
        return await self._request(
            endpoint=endpoint_user_avatar(user_id),
            method=HTTPMethod.POST,
            file=file,
        )

    async def delete_user_avatar(self, user_id: int) -> Any:
        return await self._request(
            endpoint=endpoint_user_avatar(user_id), method=HTTPMethod.DELETE
        )

    async def get_user_profile_posts(
        self, user_id: int, params: UserProfilePostsGetParams | None = None
    ) -> Any:
        return await self._request(
            endpoint=endpoint_user_profile_posts(user_id),
            method=HTTPMethod.GET,
            query_params=params,
        )

    # ============================================================================
    # ACTIONS
    # ============================================================================

    async def get_demote_groups(self) -> Any:
        return await self._request(
            endpoint=endpoint_demote, method=HTTPMethod.GET
        )

    async def demote_user(self, user_id: int, params: UserDemoteParams) -> Any:
        return await self._request(
            endpoint=endpoint_demote_user(user_id),
            method=HTTPMethod.POST,
            body_params=params,
        )

    async def get_promote_groups(self) -> Any:
        return await self._request(
            endpoint=endpoint_promote, method=HTTPMethod.GET
        )

    async def promote_user(
        self, user_id: int, params: UserPromoteParams
    ) -> Any:
        return await self._request(
            endpoint=endpoint_promote_user(user_id),
            method=HTTPMethod.POST,
            body_params=params,
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from pydantic import BaseModel, Field

from ...types.discussion_state import DiscussionStateEnum
from ..attachment import Attachment
from ..user import User

if TYPE_CHECKING:
    from ..thread import Thread

__all__ = ("Post",)


class Post(BaseModel):
    post_id: int
    thread_id: int
    user_id: int
    username: str
    message: str
    user: User | None = Field(alias="User", default=None)

    is_first_post: bool
    is_last_post: bool
    is_unread: bool
    message_parsed: str | None = None
    can_edit: bool
    can_soft_delete: bool
    can_hard_delete: bool
    can_react: bool
    can_view_attachments: bool | None = None
    view_url: str
    thread: Thread | None = Field(alias="Thread", default=None)
    attachments: list[Attachment] | None = Field(
        alias="Attachments", default=None
    )
    is_reacted_to: bool
    visitor_reaction_id: int | None = None
    vote_score: int | None = None
    can_content_vote: bool | None = None
    allowed_content_vote_types: list[str] | None = None
    is_content_voted: bool | None = None
    visitor_content_vote: str | None = None
    post_date: int
    message_state: DiscussionStateEnum
    attach_count: int
    warning_message: str
    position: int
    last_edit_date: int
    reaction_score: int



from enum import Enum

__all__ = ("ContentTypeEnum",)


class ContentTypeEnum(Enum):
    POST = "post"
    TROPHY = "trophy"
    USER = "user"
    THREAD = "thread"
    USERNAME_CHANGE = "username_change"
    STR = "str"
    CONTACT = "contact"
    CONVERSATION_MESSAGE = "conversation_message"



from pydantic import BaseModel

__all__ = ("LatestUser", "Online", "Totals")


class Totals(BaseModel):
    thread: int
    messages: int
    users: int


class LatestUser(BaseModel):
    user_id: int
    username: str
    register_date: int


class Online(BaseModel):
    total: int
    members: int
    guests: int
